<?php
namespace tmc\revisionmanager\src\AdminPages\PageSettings;

/**
 * @author jakubkuranda@gmail.com
 * Date: 15.03.2018
 * Time: 14:14
 */

use shellpress\v1_2_0\src\Shared\AdminPageFramework\AdminPageTab;
use tmc\revisionmanager\src\App;

class TabTools extends AdminPageTab {

    /**
     * Declaration of current element.
     */
    public function setUp() {

        //  ----------------------------------------
        //  Definition
        //  ----------------------------------------

        $this->pageFactory->addInPageTab(
            array(
                'page_slug'     =>  $this->pageSlug,
                'tab_slug'      =>  $this->tabSlug,
                'title'         =>  __( 'Tools', 'rm_tmc' ),
            )
        );

        //  ----------------------------------------
        //  Filters
        //  ----------------------------------------

        add_filter( 'fields_' . $this->pageFactoryClassName,                        array( $this, '_f_redefineFields_importexport' ) );
        add_filter( 'validation_' . $this->pageSlug . '_' . $this->tabSlug,         array( $this, '_f_updateKeyButtonCallback' ), 10, 2 );

    }

    /**
     * Called while current component is loaded.
     */
    public function load() {

        //  ----------------------------------------
        //  Sections
        //  ----------------------------------------

        $this->pageFactory->addSettingSections(
            array(
                'section_id'            =>  'importexport',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'order'                 =>  5,
                'title'                 =>  __( 'Import / Export', 'rm_tmc' ),
                'class'                 =>  'section-grid',
                'description'           =>  array(
                    __( 'Share your settings between many Wordpress installations ( multisite ).', 'rm_tmc' )
                )
            ),
            array(
                'section_id'            =>  'license',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'order'                 =>  10,
                'title'                 =>  __( 'License', 'rm_tmc' ),
                'description'           =>  array(
                    __( 'You can use premium features and extended support.', 'rm_tmc' ),
                    __( 'Some features require external server to provide seamless functionality.', 'rm_tmc' ),
                ),
                'class'                 =>  'section-grid'
            )
        );

        //  ----------------------------------------
        //  Fields
        //  ----------------------------------------

        $this->pageFactory->addSettingFields(
            'importexport',   //  section_id
            array(
                'field_id'              =>  'exportSettings',
                'title'                 =>  __( 'Export settings', 'rm_tmc' ),
                'type'                  =>  'export',
                'file_name'             =>  sprintf( 'Revision Manager TMC options - %1$s.txt', current_time( 'mysql' ) ),
                'data'                  =>  App::s()->options->get(),
                'label'                 =>  __( 'Download all settings', 'rm_tmc' )
            ),
            array(
                'field_id'              =>  'importSettings',
                'title'                 =>  __( 'Import settings', 'rm_tmc' ),
                'type'                  =>  'import',
                'option_key'            =>  App::s()->options->getOptionsKey(),
                'label'                 =>  __( 'Load settings from file', 'rm_tmc' ),
                'is_merge'              =>  true,
                'description'           =>  __( 'Pick file exported from other WordPress installation.', 'rm_tmc' )
            )
        );

        $this->pageFactory->addSettingFields(
            'license',  //  section_id
            array(
                'field_id'          =>  'key',
                'type'              =>  'text',
                'title'             =>  __( 'Key', 'tmc_fbleads' ),
                'attributes'            =>  array(
                    'class'                 =>  'regular-text',
                ),
                'after_input'       =>  sprintf( ' <input type="submit" class="button" name="updateKeySubmit" value="%1$s">', __( 'Update key', 'tmc_fbleads' ) ),
                'after_field'       =>  $this->getLicenseStatusHtml()
            ),
            array(
                'field_id'          =>  'keyExpiryDatetime',
                'type'              =>  'text',
                'title'             =>  'keyExpiryDatetime',
                'hidden'            =>  true,
                'attributes'        =>  array(
                    'disabled'          =>  'disabled'
                )
            ),
            array(
                'field_id'          =>  'lastCheckDatetime',
                'type'              =>  'text',
                'title'             =>  'lastCheckDatetime',
                'hidden'            =>  true,
                'attributes'        =>  array(
                    'disabled'          =>  'disabled'
                )
            ),
            array(
                'field_id'          =>  'keyStatus',
                'type'              =>  'text',
                'title'             =>  'keyStatus',
                'hidden'            =>  true,
                'attributes'        =>  array(
                    'disabled'          =>  'disabled'
                )
            ),
            array(
                'field_id'          =>  'isKeyCorrect',
                'type'              =>  'radio',
                'title'             =>  'isKeyCorrect',
                'label'             =>  array(
                    '1'                 =>  'yes',
                    '0'                 =>  'no'
                ),
                'hidden'            =>  true
            )
        );

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================


    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineFields_importexport( $fields ) {

        if( ! App::i()->license->isActive() ){

            $fields['importexport']['exportSettings']['attributes']['disabled'] = 'disabled';
            $fields['importexport']['exportSettings']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
                App::TMC_SHOP_URL,
                __( 'You can export current settings to file in PRO version.', 'rm_tmc' )
            );

            $fields['importexport']['importSettings']['attributes']['disabled'] = 'disabled';
            $fields['importexport']['importSettings']['description'] = null;
            $fields['importexport']['importSettings']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
                App::TMC_SHOP_URL,
                __( 'You can import settings from other WP in PRO version.', 'rm_tmc' )
            );

        }

        return $fields;

    }

    /**
     * Called on form validation.
     * Performs key update if there is change in key value.
     *
     * @param array $newOptions
     * @param array $oldOptions
     *
     * @return array
     */
    public function _f_updateKeyButtonCallback( $newOptions, $oldOptions ) {

        if(
            $newOptions['license']['key'] != $oldOptions['license']['key']
            || isset( $_POST['updateKeySubmit'] ) && ! empty( $_POST['updateKeySubmit'] )
        ){

            $key = $newOptions['license']['key'];

            App::i()->license->setKey( $key );
            App::i()->license->performRemoteKeyUpdate();

            $this->pageFactory->setSettingNotice( __( 'License key has been updated.', 'tmc_fbleads' ), 'updated' );

            //  ShellPress options are older than this fresh submitted data.
            //  Actions are performed on ShellPress internal storage, so we return it here for update.

            return App::s()->options->get();

        }

        return $newOptions;

    }

    //  ================================================================================
    //  FUNCTIONALITY
    //  ================================================================================

    /**
     * Returns HTML of license status.
     *
     * @return string
     */
    protected function getLicenseStatusHtml() {

        $html = '';

        if( App::i()->license->getKey() ){

            $keyStatus          = App::i()->license->getKeyStatus();
            $keyExpiryDatetime  = App::i()->license->getKeyExpiryDatetime();
            $keyIsActive        = App::i()->license->isActive();

            if( $keyStatus ){
                if( $keyIsActive ){
                    $html .= sprintf( '<div style="clear: both; color: #16a085;">%1$s</div>', $keyStatus );
                } else {
                    $html .= sprintf( '<div style="clear: both; color: #e74c3c;">%1$s</div>', $keyStatus );
                }
            }

            if( $keyExpiryDatetime && $keyIsActive ){
                $html .= sprintf( '<div style="clear: both; color: #16a085;">%1$s %2$s</div>', __( 'Valid until:', 'tmc_fbleads' ), $keyExpiryDatetime );
            }

        }

        return $html;

    }

}