<?php
namespace tmc\revisionmanager\src\Components;

/**
 * @author jakubkuranda@gmail.com
 * Date: 08.03.2018
 * Time: 13:03
 */

use tmc\revisionmanager\src\App;

class Settings {

    /**
     * Returns slugs of chosen supported post types.
     * It supports both radio and checkbox method of saved data.
     *
     * @return string[]
     */
    public function getChosenPostTypesSlugs() {

        /** @var string|array $option */
        $option = App::s()->options->get( 'postTypes/chosen', array() );
        $slugs  = array();

        if( ! empty( $option ) && is_array( $option ) ){    //  Probably checkbox option

            foreach( $option as $slug => $isSelected ){
                if( $isSelected ){

                    $slugs[] = $slug;

                }
            }

        } elseif( ! empty( $option ) ) {                    //  Probably radio option

            $slugs[] = $option;

        }

        //  Test for accepted usage.

        if( ! App::i()->license->isActive() ){
            if( in_array( 'post', $slugs ) ){
                return array( 'post' );
            } else {
                return array();
            }
        }

        return $slugs;

    }

    /**
     * @return string|null
     */
    public function getCapabilityForCopyCreation() {

        return App::s()->options->get( 'capabilities/capCopy' );

    }

    /**
     * @return string|null
     */
    public function getCapabilityForAcceptingChanges() {

        return App::s()->options->get( 'capabilities/capAccept' );

    }

    /**
     * @return string|null
     */
    public function getRoleForNotifications() {

        return App::s()->options->get( 'capabilities/roleNotification' );

    }

    /**
     * @return string[]
     */
    public function getExcludedEmailsFromNotifications() {

        $unfilteredEmails = (string) App::s()->options->get( 'capabilities/excludedEmails', '' );

        //  Remove whitespaces

        $filteredEmails = preg_replace( '/\s*/m', '', $unfilteredEmails );
        $filteredEmails = str_replace ( ' ', '', $filteredEmails );

        //  Turn string to array

        $emails = (array) explode( ',', $filteredEmails );  //  Explode emails by comma

        return $emails;

    }

    /**
     * @return string|null
     */
    public function getNotificationSubject() {

        return App::s()->options->get( 'notifications/title' );

    }

    /**
     * @return string|null
     */
    public function getNotificationContent() {

        return App::s()->options->get( 'notifications/content' );

    }

}